#
# Copyright (c) 2008-2017 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from persistent import Persistent

# import interfaces
from hypatia.interfaces import ICatalog
from pyams_content.shared.view.interfaces import IWfView, IViewSettings, IViewInternalReferencesSettings, \
    IViewQueryParamsExtension, IViewQueryFilterExtension, VIEW_REFERENCES_SETTINGS_KEY, ALWAYS_REFERENCE_MODE
from pyams_sequence.interfaces import ISequentialIdInfo

# import packages
from hypatia.catalog import CatalogQuery
from hypatia.query import Any, NotEq
from pyams_catalog.query import CatalogResultSet, and_
from pyams_content.workflow import VISIBLE_STATES
from pyams_utils.adapter import adapter_config, ContextAdapter, get_annotation_adapter
from pyams_utils.registry import get_utility
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(IViewInternalReferencesSettings)
class ViewInternalReferencesSettings(Persistent, Contained):
    """View internal references settings"""

    references = FieldProperty(IViewInternalReferencesSettings['references'])
    references_mode = FieldProperty(IViewInternalReferencesSettings['references_mode'])
    exclude_context = FieldProperty(IViewInternalReferencesSettings['exclude_context'])

    @property
    def is_using_context(self):
        return False


@adapter_config(context=IWfView, provides=IViewInternalReferencesSettings)
@adapter_config(name='references', context=IWfView, provides=IViewSettings)
def view_internal_references_settings_factory(view):
    """View internal references settings factory"""
    return get_annotation_adapter(view, VIEW_REFERENCES_SETTINGS_KEY, ViewInternalReferencesSettings,
                                  name='++view:references++')


@adapter_config(name='references', context=IWfView, provides=IViewQueryParamsExtension)
class ViewThemesQueryParamsExtension(ContextAdapter):
    """View internal references query params extension"""

    weight = 50

    def get_params(self, context):
        settings = IViewInternalReferencesSettings(self.context)
        params = None
        # check themes
        if settings.exclude_context:
            sequence = ISequentialIdInfo(context, None)
            if sequence is not None:
                oid = sequence.hex_oid
                catalog = get_utility(ICatalog)
                params = and_(params, NotEq(catalog['oid'], oid))
        return params


@adapter_config(name='references', context=IWfView, provides=IViewQueryFilterExtension)
class ViewInternalReferencesQueryFilterExtension(ContextAdapter):
    """View internal references filter extension"""

    weight = 999

    def filter(self, context, items):
        settings = IViewInternalReferencesSettings(self.context)
        if not settings.references:
            return items
        if (not items) or (settings.references_mode == ALWAYS_REFERENCE_MODE):
            catalog = get_utility(ICatalog)
            params = Any(catalog['oid'], settings.references) & \
                Any(catalog['workflow_state'], VISIBLE_STATES)
            items.prepend(CatalogResultSet(CatalogQuery(catalog).query(params)))
        return items
